@@ -173,6 +173,7 @@ from   foliage.folio import Folio
 from   foliage.list_tab import ListTab
 from   foliage.lookup_tab import LookupTab
 from   foliage.other_tab import OtherTab
+from   foliage.clean_tab import CleanTab
 from   foliage.system_widget import SystemWidget
 from   foliage.ui import quit_app, reload_page, confirm, notify, inside_pyinstaller_app
 from   foliage.ui import note_info, note_warn, note_error, tell_success, tell_failure
@@ -186,7 +187,7 @@ from   foliage.ui import close_splash_screen
 _DIRS = AppDirs('Foliage', 'CaltechLibrary')
 '''Platform-specific directories for Foliage data.'''
 
-_TABS = [LookupTab(), ChangeTab(), DeleteTab(), ListTab(), OtherTab()]
+_TABS = [LookupTab(), ChangeTab(), DeleteTab(), CleanTab(), ListTab(), OtherTab()]
 '''List of tabs making up the Foliage application.'''
 
 
@@ -408,16 +409,8 @@ def foliage_page(widget):
         notify('No network -- cannot proceed.')
         quit_app(ask_confirm = False)
 
-    # Work through our options to get (or ask for) credentials.
-    if not credentials_complete(credentials_from_env()):
-        creds = credentials_from_user(initial_creds = credentials_from_env())
-        if not credentials_complete(creds):
-            notify('Unable to proceed without complete credentials. Quitting.')
-            quit_app(ask_confirm = False)
-        use_credentials(creds)
-    if not Folio().credentials_valid():
-        notify('Invalid FOLIO credentials. Quitting.')
-        quit_app(ask_confirm = False)
+    # Make sure we have valid FOLIO credentials.
+    check_credentials()
 
     # Create a single dict from all the separate pin_watchers dicts.
     watchers  = dict(ChainMap(*[tab.pin_watchers() for tab in _TABS]))
@@ -470,7 +463,7 @@ def config_debug(debug_arg):
                     note_error(f'Can\'t write debug ouput in {log_dir}')
                     exit()
             faulthandler.enable()
-            if not os.name == 'nt':     # Can't use next part on Windows.
+            if os.name != 'nt':         # Can't use next part on Windows.
                 import signal
                 from boltons.debugutils import pdb_on_signal
                 pdb_on_signal(signal.SIGUSR1)
@@ -628,6 +621,32 @@ def warn_if_demo_mode():
     else:
         log('Demo mode not in effect')
 
+
+def check_credentials():
+    '''Check that the credentials we have are complete and valid.
+    If they are not, ask the user if they want to edit them.
+    '''
+    def edit_and_use_credentials():
+        creds = credentials_from_user(initial_creds = credentials_from_env())
+        if not credentials_complete(creds):
+            notify('Unable to proceed without complete credentials. Quitting.')
+            quit_app(ask_confirm = False)
+        use_credentials(creds)
+
+    if not credentials_complete(credentials_from_env()):
+        edit_and_use_credentials()
+    if not Folio().credentials_valid():
+        # FOLIO might have invalidated users' tokens.
+        if confirm('The FOLIO token may have expired, or else the given '
+                   + ' credentials are invalid. Click "OK" to review the '
+                   + ' credentials and try to regenerate the token, or click '
+                   + ' "Cancel" to quit Foliage now.'):
+            edit_and_use_credentials()
+        else:
+            notify('Invalid FOLIO credentials, or expired token.')
+            quit_app(ask_confirm = False)
+
+
 
 # Main entry point.
 # .............................................................................
