@@ -61,6 +61,9 @@ from   foliage.exceptions import *
 # Internal constants.
 # .............................................................................
 
+# Accession number prefix for this site.
+_AN_PREFIX = 'cit.oai.folio.org.fs00001057'
+
 # Number of times we retry an api call that return an HTTP error.
 _MAX_RETRY = 3
 
@@ -463,7 +466,10 @@ class Folio():
             return self._kind_cache[id]
 
         id_kind = IdKind.UNKNOWN
-        if (isint(id) and id.startswith('350')) or id.startswith('nobarcode'):
+        if ((isint(id) and id.startswith('350'))
+            or id.startswith('nobarcode')
+            or id.startswith('temp-') or id.startswith('TEMP-')
+            or (isint(id) and 1 <= int(id) <= 200)):
             log(f'recognized {id} as an item barcode')
             id_kind = IdKind.ITEM_BARCODE
         elif id.startswith('it') and id[2].isdigit():
@@ -472,7 +478,7 @@ class Folio():
         elif id.startswith('ho') and id[2].isdigit():
             log(f'recognized {id} as an holdings hrid')
             id_kind = IdKind.HOLDINGS_HRID
-        elif id.startswith('clc') and '.' in id:
+        elif id.startswith(_AN_PREFIX):
             log(f'recognized {id} as an accession number')
             id_kind = IdKind.ACCESSION
         elif id.count('-') > 2:
@@ -499,6 +505,9 @@ class Folio():
             # Most hrid's will follow the pattern above, so try other cases 1st.
             folio_searches = [
                 ('/users?query=barcode=',                   IdKind.USER_BARCODE),
+                # Caltech ID's are same as user barcodes with a leading 000.
+                # Let's try to help people who type in a CITUID without the 0's.
+                ('/users?query=barcode=000',                IdKind.USER_BARCODE),
                 ('/instance-storage/instances?query=hrid=', IdKind.INSTANCE_HRID),
                 ('/item-storage/items?query=hrid=',         IdKind.ITEM_HRID),
                 ('/holdings-storage/holdings?query=hrid=',  IdKind.HOLDINGS_HRID),
@@ -549,7 +558,7 @@ class Folio():
         associated with "id".
         '''
         use_inv = 'using inventory API' if use_inventory else ''
-        log(f'getting {requested} record(s) for {id_kind} id {id} {use_inv}')
+        log(f'getting {requested} record(s) for {id_kind} {id} {use_inv}')
 
         def record_list(kind, key, response):
             if not response or not response.text or response.status_code == 404:
@@ -772,19 +781,9 @@ class Folio():
                 if open_loans_only:
                     loans = [ln for ln in loans if ln.data['status']['name'] == 'Open']
                 return loans
-            elif id_kind == IdKind.ITEM_BARCODE:
-                # Can't seem to use barcodes directly in loan-storage.
-                log(f'need to find item id for item barcode {id}')
-                records = self.related_records(id, IdKind.ITEM_BARCODE, 'item',
-                                               use_inventory, open_loans_only)
-                if not records:
-                    return []
-                item_id = records[0].id
-                return self.related_records(item_id, IdKind.ITEM_ID, 'loan',
-                                            use_inventory, open_loans_only)
-            elif id_kind == IdKind.ITEM_HRID:
-                log(f'need to find item id for item hrid {id}')
-                records = self.related_records(id, IdKind.ITEM_HRID, 'item',
+            elif id_kind in [IdKind.ITEM_BARCODE, IdKind.ITEM_HRID]:
+                log(f'need to find item id for {id_kind} {id}')
+                records = self.related_records(id, id_kind, 'item',
                                                use_inventory, open_loans_only)
                 if not records:
                     return []
@@ -804,49 +803,35 @@ class Folio():
                 if open_loans_only:
                     loans = [ln for ln in loans if ln.data['status']['name'] == 'Open']
                 return loans
-            elif id_kind == IdKind.INSTANCE_HRID:
-                # Get the instance record & do this again with the instance id.
-                records = self.related_records(id, IdKind.INSTANCE_HRID, 'instance',
+            elif id_kind in [IdKind.INSTANCE_HRID, IdKind.ACCESSION]:
+                # Get the instance record & do this again with the instance id,
+                # because we solved that case in the code above.
+                records = self.related_records(id, id_kind, 'instance',
                                                use_inventory, open_loans_only)
                 if not records:
                     return []
                 instance_id = records[0].id
                 return self.related_records(instance_id, IdKind.INSTANCE_ID, 'loan',
                                             use_inventory, open_loans_only)
-            elif id_kind == IdKind.ACCESSION:
-                # Get the instance record & do this again with the instance id.
-                records = self.related_records(id, IdKind.ACCESSION, 'instance',
-                                               use_inventory, open_loans_only)
-                if not records:
-                    return []
-                instance_id = records[0].id
-                return self.related_records(instance_id, IdKind.INSTANCE_ID, 'loan',
-                                            use_inventory, open_loans_only)
-            elif id_kind == IdKind.HOLDINGS_ID:
-                holdings = self.related_records(id, IdKind.HOLDINGS_ID, 'holdings',
-                                                use_inventory, open_loans_only)
-                if not holdings:
-                    return []
-                instance_id = holdings[0].data['instanceId']
-                return self.related_records(instance_id, IdKind.INSTANCE_ID, 'loan',
-                                            use_inventory, open_loans_only)
-            elif id_kind == IdKind.HOLDINGS_HRID:
-                holdings = self.related_records(id, IdKind.HOLDINGS_HRID, 'holdings',
-                                                use_inventory, open_loans_only)
-                if not holdings:
-                    return []
-                instance_id = holdings[0].data['instanceId']
-                return self.related_records(instance_id, IdKind.INSTANCE_ID, 'loan',
-                                            use_inventory, open_loans_only)
+            elif id_kind in [IdKind.HOLDINGS_ID, IdKind.HOLDINGS_HRID]:
+                # Can't go straight from holdings records to loans. Get items
+                # under this holdings record, then get loans on those items.
+                items = self.related_records(id, id_kind, 'item',
+                                             use_inventory, open_loans_only)
+                loans = []
+                for item in items:
+                    loans += self.related_records(item.id, IdKind.ITEM_ID, 'loan',
+                                                  use_inventory, open_loans_only)
+                return loans
             else:
                 raise RuntimeError(f'Unsupported combo: searching for {requested} by {id_kind.value}')
 
         elif requested == RecordKind.USER:
             if id_kind == IdKind.USER_ID:
-                endpoint = f'/users/{id}'
+                endpoint = f'/users/{id.zfill(10)}'
                 data_extractor = partial(record_list, RecordKind.USER, None)
             elif id_kind == IdKind.USER_BARCODE:
-                endpoint = f'/users?query=barcode={id}'
+                endpoint = f'/users?query=barcode={id.zfill(10)}'
                 data_extractor = partial(record_list, RecordKind.USER, 'users')
             elif id_kind == IdKind.ITEM_ID:
                 records = self.related_records(id, IdKind.ITEM_ID, 'loan',
@@ -1070,9 +1055,8 @@ class Folio():
 # Misc. utilities
 # .............................................................................
 
-def instance_id_from_accession(an):
-    start = an.find('.')
-    id_part = an[start + 1:]
+def instance_id_from_accession(accession_number):
+    id_part = accession_number[len(_AN_PREFIX) + 1:]
     return id_part.replace('.', '-')
 
 
@@ -1080,7 +1064,7 @@ def unique_identifiers(text):
     '''Return a list of identifiers found in the text after some cleanup.'''
     lines = text.splitlines()
     ids = flattened(re.split(r'\s+|,+|;+|:+', line) for line in lines)
-    ids = [id.strip(r'''.'":''') for id in ids]
+    ids = [id.strip(r'''.'":?!/''') for id in ids]
     ids = [id for id in ids if not any(c in id for c in r'!@#$%^&*=\/')]
     ids = [id for id in ids if any(c.isnumeric() for c in id)]
     return unique(filter(None, ids))
