@@ -23,7 +23,7 @@ from   pywebio.output import use_scope, set_scope, clear, remove, put_warning
 from   pywebio.output import put_success, put_info, put_table, put_grid, span
 from   pywebio.output import put_tabs, put_image, put_scrollable, put_code, put_link
 from   pywebio.output import put_processbar, set_processbar, put_loading
-from   pywebio.output import put_column, put_scope, clear_scope
+from   pywebio.output import put_column, put_scope, clear_scope, get_scope
 from   pywebio.pin import pin, pin_wait_change, put_input, put_actions
 from   pywebio.pin import put_textarea, put_radio, put_checkbox, put_select
 from   pywebio.session import run_js, eval_js
@@ -50,7 +50,7 @@ class LookupTab(FoliageTab):
         return {}
 
 
-# Tab body.
+# Tab layout.
 # .............................................................................
 
 def tab_contents():
@@ -107,6 +107,8 @@ def tab_contents():
 # Tab implementation.
 # .............................................................................
 
+_interrupted = False
+_running = False
 _last_textbox = ''
 _last_results = {}
 _last_kind = None
@@ -114,6 +116,13 @@ _last_inventory_api = True
 _last_open_loans = True
 _location_map = None
 
+
+def load_file():
+    log(f'user requesting file upload')
+    if (contents := user_file('Upload a file containing identifiers')):
+        pin.textbox_find = contents
+
+
 def init_location_map():
     global _location_map
     if _location_map is None:
@@ -127,10 +136,12 @@ def inputs_are_unchanged():
     global _last_kind
     global _last_inventory_api
     global _last_open_loans
-    return (pin.textbox_find == _last_textbox
-            and pin.select_kind == _last_kind
-            and pin.inventory_api == _last_inventory_api
-            and pin.open_loans == _last_open_loans)
+    unchanged = (pin.textbox_find == _last_textbox
+                 and pin.select_kind == _last_kind
+                 and pin.inventory_api == _last_inventory_api
+                 and pin.open_loans == _last_open_loans)
+    log(f'field values are considered {"unchanged" if unchanged else "changed"}')
+    return unchanged
 
 
 def clear_tab():
@@ -146,19 +157,62 @@ def clear_tab():
 
 
 def stop():
+    '''Stop an ongoing lookup by setting the _interrupted flag.'''
+    global _interrupted
     global _last_textbox
     log(f'stopping')
-    interrupt()
-    stop_processbar()
+    _interrupted = True
     _last_textbox = ''
+    stop_processbar()
+    interrupt()
+    with use_scope('output'):
+        tell_warning('**Stopping** ...')
 
 
-def load_file():
-    log(f'user requesting file upload')
-    if (contents := user_file('Upload a file containing identifiers')):
-        pin.textbox_find = contents
+def reset():
+    # Reset to state where we can run new operations.
+    global _interrupted
+    _interrupted = False
+    reset_interrupts()
 
 
+def enable_lookup_button(state):
+    '''Enable the "look up records" button if True, disable if False.'''
+    action = 'removeClass' if state else 'addClass'
+    eval_js(f'''$("button:contains('Look up records')").{action}("disabled-button");''')
+
+
+def wait_if_running():
+    '''Check if the run state is running; if it is, wait until it changes.'''
+    # The _running variable is set by do_find() when it's finished, and it's
+    # set even if it gets interrupted. This is what we cue from.
+    global _running
+    if not _running:
+        return
+    enable_lookup_button(False)
+    stop()
+    # Wait in case an ongoing lookup is running, but don't wait forever.
+    wait_count = 10
+    while _running and wait_count > 0:
+        # If the user clicks multiple times rapidly, the exception raised for
+        # interrupts starts to cascade. Wrap with a try-except to avoid this.
+        try:
+            wait(1)
+            wait_count -= 1
+        except Interrupted:
+            continue
+    enable_lookup_button(True)
+
+
+# Summary of the basic flow of control:
+#
+# User clicks "look up records", thus invoking do_find().
+# We show progress bar & stop button while lookup is running.
+# Possible scenarios:
+#   1) process finishes normally
+#   2) user clicks stop button
+#   3) user clicks "look up records" button while lookup is running
+
 def do_find():
     global _last_results
     global _last_textbox
@@ -166,7 +220,11 @@ def do_find():
     global _last_inventory_api
     global _last_open_loans
     global _location_map
-    log(f'do_find invoked')
+    global _interrupted
+    global _running
+    log('do_find invoked')
+    wait_if_running()
+    reset()
     # Normally we'd want to find out if they input any identifiers, but I want
     # to detect *any* change to the input box, so this is a lower-level test.
     if not pin.textbox_find.strip():
@@ -188,8 +246,8 @@ def do_find():
     kind_wanted = pin.select_kind
     steps = len(identifiers) + 1
     folio = Folio()
-    reset_interrupts()
     init_location_map()
+    total_found = 0
     with use_scope('output', clear = True):
         put_grid([[
             put_scope('current_activity', [
@@ -201,12 +259,15 @@ def do_find():
                        onclick = lambda: stop()).style('text-align: right'),
         ]], cell_widths = '85% 15%').style(PROGRESS_BOX)
         # The staff want to see location names, so we need to get the mapping.
+        _running = True
         for count, id in enumerate(identifiers, start = 2):
+            if _interrupted:
+                break
             try:
                 # Figure out what kind of identifier we were given.
                 id_kind = folio.id_kind(id)
                 if id_kind is IdKind.UNKNOWN:
-                    tell_failure(f'Unrecognized identifier kind: **{id}**.')
+                    tell_failure(f'Unrecognized identifier: **{id}**.')
                     continue
                 if reuse_results:
                     records = _last_results.get(id)
@@ -228,9 +289,10 @@ def do_find():
                 show_index = (len(records) > 1)
                 for index, record in enumerate(records, start = 1):
                     print_record(record, id, index, show_index, pin.show_raw == 'json')
+                total_found += len(records)
             except Interrupted as ex:
                 log('stopping due to interruption')
-                break
+                _interrupted = True
             except Exception as ex:
                 import traceback
                 log('Exception info: ' + str(ex) + '\n' + traceback.format_exc())
@@ -238,25 +300,45 @@ def do_find():
                 stop_processbar()
                 return
             finally:
-                if not interrupted():
+                if not _interrupted:
                     set_processbar('bar', count/steps)
         stop_processbar()
         clear_scope('current_activity')
-        if interrupted():
+        if _interrupted:
             tell_warning('**Stopped**.')
         else:
-            summary = (f'Finished looking for {kind_wanted} records given '
-                       + pluralized('unique identifier', identifiers, True))
+            summary = (f'Found {total_found} {kind_wanted} records by looking up '
+                       + pluralized('unique identifier', identifiers, True)
+                       + '.')
             put_grid([[
                 put_markdown(summary).style('margin-top: 6px'),
                 put_button('Export', outline = True,
                            onclick = lambda: do_export(_last_results, kind_wanted),
                            ).style('text-align: right')
             ]]).style('margin: 1.5em 17px auto 17px')
+        _running = False
 
 
-def location(location_data):
+def field(record, field_name, subfield_name = None, list_joiner = ', '):
+    if field_name not in record.data:
+        return ''
+    if subfield_name:
+        if subfield_name not in record.data[field_name]:
+            return ''
+        value = record.data[field_name][subfield_name]
+    else:
+        value = record.data[field_name]
+    if isinstance(value, list) and list_joiner:
+        return list_joiner.join(str(x) for x in value)
+    else:
+        return str(value)
+
+
+def location(record, field_name):
     global _location_map
+    if field_name not in record.data:
+        return ''
+    location_data = record.data[field_name]
     if isinstance(location_data, dict):
         if 'name' in location_data:
             return f'{location_data["name"]}  ({location_data["id"]})'
@@ -267,7 +349,10 @@ def location(location_data):
     return '(unknown location)'
 
 
-def notes(notes):
+def notes(record, field_name):
+    if field_name not in record.data:
+        return ''
+    notes = record.data[field_name]
     if isinstance(notes, str):
         return notes
     elif isinstance(notes, list):
@@ -293,116 +378,116 @@ def print_record(record, identifier, index, show_index, show_raw):
         if 'title' in record.data:
             # Inventory record version.
             put_table([
-                ['Title'                     , record.data['title']],
-                ['Barcode'                   , record.data['barcode']],
-                ['Call number'               , record.data['callNumber']],
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['Effective location'        , location(record.data['effectiveLocation'])],
-                ['Permanent location'        , location(record.data['permanentLocation'])],
-                ['Status'                    , record.data['status']['name']],
-                ['Tags'                      , ', '.join(t for t in record.data['tags']['tagList'])],
-                ['Notes'                     , notes(record.data['notes'])],
-                ['HRID'                      , record.data['hrid']],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                ['Title'                     , field(record, 'title')],
+                ['Barcode'                   , field(record, 'barcode')],
+                ['Call number'               , field(record, 'callNumber')],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['Effective location'        , location(record, 'effectiveLocation')],
+                ['Permanent location'        , location(record, 'permanentLocation')],
+                ['Status'                    , field(record, 'status', 'name')],
+                ['Tags'                      , field(record, 'tags', 'tagsList')],
+                ['Notes'                     , notes(record, 'notes')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
         else:
             # Storage record version.
             put_table([
-                ['Barcode'                   , record.data['barcode']],
-                ['Call number'               , record.data['itemLevelCallNumber']],
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['Effective location'        , location(record.data['effectiveLocationId'])],
-                ['Permanent location'        , location(record.data['permanentLocationId'])],
-                ['Tags'                      , ', '.join(t for t in record.data['tags']['tagList'])],
-                ['Notes'                     , notes(record.data['notes'])],
-                ['HRID'                      , record.data['hrid']],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                ['Barcode'                   , field(record, 'barcode')],
+                ['Call number'               , field(record, 'itemLevelCallNumber')],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['Effective location'        , location(record, 'effectiveLocationId')],
+                ['Permanent location'        , location(record, 'permanentLocationId')],
+                ['Tags'                      , field(record, 'tags', 'tagsList')],
+                ['Notes'                     , notes(record, 'notes')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
     elif record.kind is RecordKind.INSTANCE:
         # Caution: left-hand values contain nonbreaking spaces (invisible here).
-        if record.data['classifications']:
+        if field(record, 'classifications'):
             call_number = record.data['classifications'][0]['classificationNumber']
         else:
             call_number = ''
         if 'tags' in record.data:
             put_table([
-                ['Title'                     , record.data['title']],
+                ['Title'                     , field(record, 'title')],
                 ['Call number'               , call_number],
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['Tags'                      , ', '.join(t for t in record.data['tags']['tagList'])],
-                ['Notes'                     , notes(record.data['notes'])],
-                ['HRID'                      , record.data['hrid']],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['Tags'                      , field(record, 'tags', 'tagsList')],
+                ['Notes'                     , notes(record, 'notes')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
         else:
             put_table([
-                ['Title'                     , record.data['title']],
+                ['Title'                     , field(record, 'title')],
                 ['Call number'               , call_number],
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['HRID'                      , record.data['hrid']],
-                ['Notes'                     , notes(record.data['notes'])],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Notes'                     , notes(record, 'notes')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
     elif record.kind is RecordKind.HOLDINGS:
         # Caution: left-hand values contain nonbreaking spaces (invisible here).
         if 'effectiveLocationId' in record.data:
             put_table([
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['HRID'                      , record.data['hrid']],
-                ['Holdings type id'          , record.data['holdingsTypeId']],
-                ['Instance id'               , record.data['instanceId']],
-                ['Effective location'        , location(record.data['effectiveLocationId'])],
-                ['Permanent location'        , location(record.data['permanentLocationId'])],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Holdings type id'          , field(record, 'holdingsTypeId')],
+                ['Instance id'               , field(record, 'instanceId')],
+                ['Effective location'        , location(record, 'effectiveLocationId')],
+                ['Permanent location'        , location(record, 'permanentLocationId')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
         else:
             put_table([
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['HRID'                      , record.data['hrid']],
-                ['Holdings type id'          , record.data['holdingsTypeId']],
-                ['Instance id'               , record.data['instanceId']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['HRID'                      , field(record, 'hrid')],
+                ['Holdings type id'          , field(record, 'holdingsTypeId')],
+                ['Instance id'               , field(record, 'instanceId')],
                 ['Effective location'        , ''],
-                ['Permanent location'        , location(record.data['permanentLocationId'])],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                ['Permanent location'        , location(record, 'permanentLocationId')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
     elif record.kind is RecordKind.USER:
         # Caution: left-hand values contain nonbreaking spaces (invisible here).
         put_table([
-            ['Username'                  , record.data['username']],
-            ['Barcode'                   , record.data['barcode']],
-            [f'{record.kind.title()} id' , record.data['id']],
-            ['Patron group'              , record.data['patronGroup']],
-            ['Created'                   , record.data['metadata']['createdDate']],
-            ['Updated'                   , record.data['metadata']['updatedDate']],
+            ['Username'                  , field(record, 'username')],
+            ['Barcode'                   , field(record, 'barcode')],
+            [f'{record.kind.title()} id' , field(record, 'id')],
+            ['Patron group'              , field(record, 'patronGroup')],
+            ['Created'                   , field(record, 'metadata', 'createdDate')],
+            ['Updated'                   , field(record, 'metadata', 'updatedDate')],
         ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
     elif record.kind is RecordKind.LOAN:
         if 'userId' in record.data:
             put_table([
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['Status',                     record.data['status']['name']],
-                ['User id'                   , record.data['userId']],
-                ['Item id'                   , record.data['itemId']],
-                ['Loan date'                 , record.data['loanDate']],
-                ['Due date'                  , record.data['dueDate']],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['Status',                     field(record, 'status', 'name')],
+                ['User id'                   , field(record, 'userId')],
+                ['Item id'                   , field(record, 'itemId')],
+                ['Loan date'                 , field(record, 'loanDate')],
+                ['Due date'                  , field(record, 'dueDate')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
         else:
             put_table([
-                [f'{record.kind.title()} id' , record.data['id']],
-                ['Status',                     record.data['status']['name']],
+                [f'{record.kind.title()} id' , field(record, 'id')],
+                ['Status',                     field(record, 'status', 'name')],
                 ['User id'                   , ''],
-                ['Item id'                   , record.data['itemId']],
-                ['Loan date'                 , record.data['loanDate']],
-                ['Due date'                  , record.data['dueDate']],
-                ['Created'                   , record.data['metadata']['createdDate']],
-                ['Updated'                   , record.data['metadata']['updatedDate']],
+                ['Item id'                   , field(record, 'itemId')],
+                ['Loan date'                 , field(record, 'loanDate')],
+                ['Due date'                  , field(record, 'dueDate')],
+                ['Created'                   , field(record, 'metadata', 'createdDate')],
+                ['Updated'                   , field(record, 'metadata', 'updatedDate')],
             ]).style('font-size: 90%; margin: auto 17px 1.5em 17px')
 
 
